unit ExeInformation;
//--- Ecrit par [SFX]-Zeus : zeus@sfxteam.org 
interface
uses
  Windows, SysUtils;

Type
  TDelphiSignature=record
    Signature:array [0..3] of LongWord;
    Version:string;
  end;
const
  DelphiSignature:array [0..2] of TDelphiSignature=(
    (Signature:($384F3D26,$B83782C2,$034224F3,$833A9B17);Version:'Delphi Entreprise'),
    (Signature:($235D7823,$19F3A5B6,$0240F343,$C711D126);Version:'Delphi Perso'),
    (Signature:($98DF8CA2,$793A3C7B,$093F7126,$17252A0F);Version:'Delphi Pro')
    );

Type
  PPackageInfoHeader = ^TPackageInfoHeader;
  TPackageInfoHeader = packed record
    Flags: Cardinal;
    RequiresCount: Integer;
  end;
  PDVCLAL =^TDVCLAL;
  TDVCLAL = array[0..3] of LongWord;

  TExeInfo = class
  private
    FCompilateur: string;
    FCompilateurVersion:string;
    FCompilateurIdVersion:string;
    FIsDelphi:Boolean;
  public
    constructor Create(Filename:TFileName);
    procedure GetExeInfo(Filename:TFileName);
    property Compilateur:string read FCompilateur;
    property CompilateurVersion:string read FCompilateurVersion;
    property CompilateurIdVersion:string read FCompilateurIdVersion;
    property ExeDelphi:boolean read FIsDelphi;
  end;

implementation
{ TExeInfo }

constructor TExeInfo.Create(Filename: TFileName);
begin
  GetExeInfo(Filename);
end;


procedure TExeInfo.GetExeInfo(Filename: TFileName);
var
  HLib,Hres:THandle;
  i:integer;
  PackageInfoHeader:PPackageInfoHeader;
  DVCLAL:PDVCLAL;
begin
  FIsDelphi:=false;
  FCompilateurVersion:='';
  FCompilateurIdVersion:='';
  FCompilateur:='Inconnu';
  HLib := LoadLibraryEx(PChar(Filename), 0,LOAD_LIBRARY_AS_DATAFILE);
  if HLib>0 then
  begin
    Hres:=FindResource(HLib,'DVCLAL',RT_RCDATA);
    if Hres>0 then
    begin
      FCompilateurVersion:='Signature inconnue';
      FIsDelphi:=true;
      DVCLAL:=Pointer(LoadResource(HLib,Hres));
      for i:=0 to 3 do FCompilateurIdVersion:=FCompilateurIdVersion+' '+IntToHex(DVCLAL^[i],2);

      for i:=0 to 2 do
      begin
        if CompareMem(DVCLAL,@DelphiSignature[i].Signature,16) then
        begin
          FCompilateurVersion:=DelphiSignature[i].Version;
        end;
      end;
    end
    else
    begin
      FIsDelphi:=false;
    end;

    Hres:=FindResource(Hlib,'PACKAGEINFO',RT_RCDATA);
    if Hres>0 then
    begin
      PackageInfoHeader:=Pointer(LoadResource(HLib,Hres));
      case ((PackageInfoHeader^.Flags and $0C000000) shr 26) of
        0 : FCompilateur:='Pre-V4';
        1 : FCompilateur:='undefined';
        2 : FCompilateur:='C++';
        3 : FCompilateur:='Delphi';
      end;
    end;
  end;
  FreeLibrary(Hlib);
end;

//----------------------------------------
end.
